@@ -66,6 +66,7 @@ namespace Oxygen
 
         _midColorCache.clear();
 
+        _dockWidgetButtonCache.clear();
         _progressBarCache.clear();
         _cornerCache.clear();
         _selectionCache.clear();
@@ -94,6 +95,7 @@ namespace Oxygen
         _holeCache.setMaxCacheSize( value );
         _scrollHandleCache.setMaxCacheSize( value );
 
+        _dockWidgetButtonCache.setMaxCost( value );
         _progressBarCache.setMaxCost( value );
         _cornerCache.setMaxCost( value );
         _selectionCache.setMaxCost( value );
@@ -164,10 +166,10 @@ namespace Oxygen
     }
 
     //______________________________________________________________________________
-    QPixmap StyleHelper::windecoButton( const QColor& color, bool pressed, int size )
+    QPixmap StyleHelper::dockWidgetButton( const QColor& color, bool pressed, int size )
     {
         const quint64 key( ( quint64( color.rgba() ) << 32 ) | ( size << 1 ) | quint64( pressed ) );
-        QPixmap *pixmap = windecoButtonCache().object( key );
+        QPixmap *pixmap = _dockWidgetButtonCache.object( key );
 
         if ( !pixmap )
         {
@@ -195,7 +197,7 @@ namespace Oxygen
                 p.end();
             }
 
-            windecoButtonCache().insert( key, pixmap );
+            _dockWidgetButtonCache.insert( key, pixmap );
         }
 
         return *pixmap;
@@ -1209,40 +1211,39 @@ namespace Oxygen
             QPixmap pixmap( 32+16, height );
             pixmap.fill( Qt::transparent );
 
-            QRect r( pixmap.rect().adjusted( 0, 0, -1, -1 ) );
+            QRectF r( pixmap.rect() );
 
             QPainter p( &pixmap );
             p.setRenderHint( QPainter::Antialiasing );
-            p.translate( .5, .5 );
+
+            // items with custom background brushes always have their background drawn
+            // regardless of whether they are hovered or selected or neither so
+            // the gradient effect needs to be more subtle
 
             {
-
-                // background
-                QPainterPath path;
-                path.addRoundedRect( r, rounding, rounding );
-
-                // items with custom background brushes always have their background drawn
-                // regardless of whether they are hovered or selected or neither so
-                // the gradient effect needs to be more subtle
+                // fill
                 const int lightenAmount( custom ? 110 : 130 );
                 QLinearGradient gradient( 0, 0, 0, r.bottom() );
                 gradient.setColorAt( 0, color.lighter( lightenAmount ) );
                 gradient.setColorAt( 1, color );
 
-                p.setPen( QPen( color, 1 ) );
+                p.setPen( Qt::NoPen );
                 p.setBrush( gradient );
-                p.drawPath( path );
+                p.drawRoundedRect( r, rounding+0.5, rounding+0.5 );
 
             }
 
             {
-                // contrast pixel
-                QPainterPath path;
-                path.addRoundedRect( r.adjusted( 1, 1, -1, -1 ), rounding - 1, rounding - 1 );
-                p.strokePath( path, QPen( QColor( 255, 255, 255, 64 ), 1 ) );
-            }
+                // contrast
+                QLinearGradient gradient( 0, 0, 0, r.bottom() );
+                gradient.setColorAt( 0, color );
+                gradient.setColorAt( 1, Qt::transparent );
 
-            p.end();
+                r.adjust( 0.5, 0.5, -0.5, -0.5 );
+                p.setPen( QPen( color, 1 ) );
+                p.setBrush( Qt::NoBrush );
+                p.drawRoundedRect( r, rounding, rounding );
+            }
 
             tileSet = new TileSet( pixmap, 8, 0, 32, height );
             _selectionCache.insert( key, tileSet );
