@@ -247,7 +247,6 @@ void Workspace::setActiveClient(Client* c, allowed_t)
     }
     pending_take_activity = NULL;
 
-    updateCurrentTopMenu();
     updateToolWindows(false);
     if (c)
         disableGlobalShortcutsForClient(c->rules()->checkDisableGlobalShortcuts(false));
@@ -263,9 +262,6 @@ void Workspace::setActiveClient(Client* c, allowed_t)
     if (active_client) {
         active_client->sl_activated();
     }
-
-    if (tilingEnabled())
-        notifyTilingWindowActivated(active_client);
     --set_active_client_recursion;
 }
 
@@ -359,7 +355,7 @@ void Workspace::takeActivity(Client* c, int flags, bool handled)
         }
         cancelDelayFocus();
     }
-    if (!(flags & ActivityFocusForce) && (c->isTopMenu() || c->isDock() || c->isSplash()))
+    if (!(flags & ActivityFocusForce) && (c->isDock() || c->isSplash()))
         flags &= ~ActivityFocus; // toplevel menus and dock windows don't take focus if not forced
     if (c->isShade()) {
         if (c->wantsInput() && (flags & ActivityFocus)) {
@@ -416,54 +412,82 @@ static inline bool isUsableFocusCandidate(Client *c, Client *prev, bool respectS
 bool Workspace::activateNextClient(Client* c)
 {
     // if 'c' is not the active or the to-become active one, do nothing
-    if (!(c == active_client
-            || (should_get_focus.count() > 0 && c == should_get_focus.last())))
+    if (!(c == active_client || (should_get_focus.count() > 0 && c == should_get_focus.last())))
         return false;
+
     closeActivePopup();
+
     if (c != NULL) {
         if (c == active_client)
             setActiveClient(NULL, Allowed);
         should_get_focus.removeAll(c);
     }
-    if (focusChangeEnabled()) {
-        if (options->focusPolicyIsReasonable()) {
-            // search the focus_chain for a client to transfer focus to,
-            Client* get_focus = NULL;
 
-            // first try to pass the focus to the (former) active clients leader
-            if (c  && (get_focus = c->transientFor()) &&
-                    isUsableFocusCandidate(get_focus, c, options->separateScreenFocus)) {
-                raiseClient(get_focus);   // also raise - we don't know where it came from
-            } else { // nope, ask the focus chain for the next candidate
-                get_focus = NULL; // reset
-                for (int i = focus_chain[ currentDesktop()].size() - 1; i >= 0; --i) {
-                    Client* ci = focus_chain[ currentDesktop()].at(i);
-                    if (isUsableFocusCandidate(ci, c, options->separateScreenFocus)) {
-                        get_focus = ci;
-                        break; // we're done
-                    }
-                }
+    // if blocking focus, move focus to the desktop later if needed
+    // in order to avoid flickering
+    if (!focusChangeEnabled()) {
+        focusToNull();
+        return true;
+    }
+
+    if (!options->focusPolicyIsReasonable())
+        return false;
+
+    Client* get_focus = NULL;
+
+    if (options->nextFocusPrefersMouse) {
+        QList<Client*>::const_iterator it = stackingOrder().constEnd();
+        while (it != stackingOrder().constBegin()) {
+            Client *client = *(--it);
+
+            // rule out clients which are not really visible.
+            // the screen test is rather superflous for xrandr & twinview since the geometry would differ -> TODO: might be dropped
+            if (!(client->isShown(false) && client->isOnCurrentDesktop() &&
+                  client->isOnCurrentActivity() && client->isOnScreen(c ? c->screen() : activeScreen())))
+                continue;
+
+            if (client->geometry().contains(QCursor::pos())) {
+                if (client != c && !client->isDesktop()) // should rather not happen, but it cannot get the focus. rest of usability is tested above
+                    get_focus = client;
+                break; // unconditional break  - we do not pass the focus to some client below an unusable one
             }
 
-            if (get_focus == NULL)   // last chance: focus the desktop
-                get_focus = findDesktop(true, currentDesktop());
+        }
+    }
 
-            if (get_focus != NULL)
-                requestFocus(get_focus);
-            else
-                focusToNull();
-        } else
-            return false;
-    } else
-        // if blocking focus, move focus to the desktop later if needed
-        // in order to avoid flickering
+    if (!get_focus) { // no suitable window under the mouse -> find sth. else
+
+        // first try to pass the focus to the (former) active clients leader
+        if (c  && (get_focus = c->transientFor()) && isUsableFocusCandidate(get_focus, c, options->separateScreenFocus)) {
+            raiseClient(get_focus);   // also raise - we don't know where it came from
+        } else {
+            // nope, ask the focus chain for the next candidate
+            get_focus = NULL; // reset from the inline assignment above
+            for (int i = focus_chain[ currentDesktop()].size() - 1; i >= 0; --i) {
+                Client* ci = focus_chain[ currentDesktop()].at(i);
+                if (isUsableFocusCandidate(ci, c, options->separateScreenFocus)) {
+                    get_focus = ci;
+                    break; // we're done
+                }
+            }
+        }
+    }
+
+    if (get_focus == NULL)   // last chance: focus the desktop
+        get_focus = findDesktop(true, currentDesktop());
+
+    if (get_focus != NULL)
+        requestFocus(get_focus);
+    else
         focusToNull();
+
     return true;
+
 }
 
 void Workspace::setCurrentScreen(int new_screen)
 {
-    if (new_screen < 0 || new_screen > numScreens())
+    if (new_screen < 0 || new_screen >= numScreens())
         return;
     if (!options->focusPolicyIsReasonable())
         return;
@@ -626,32 +650,6 @@ void Workspace::clientAttentionChanged(Client* c, bool set)
         attention_chain.removeAll(c);
 }
 
-// This is used when a client should be shown active immediately after requestFocus(),
-// without waiting for the matching FocusIn that will really make the window the active one.
-// Used only in special cases, e.g. for MouseActivateRaiseandMove with transparent windows,
-bool Workspace::fakeRequestedActivity(Client* c)
-{
-    if (should_get_focus.count() > 0 && should_get_focus.last() == c) {
-        if (c->isActive())
-            return false;
-        c->setActive(true);
-        return true;
-    }
-    return false;
-}
-
-void Workspace::unfakeActivity(Client* c)
-{
-    if (should_get_focus.count() > 0 && should_get_focus.last() == c) {
-        // TODO this will cause flicker, and probably is not needed
-        if (last_active_client != NULL)
-            last_active_client->setActive(true);
-        else
-            c->setActive(false);
-    }
-}
-
-
 //********************************************
 // Client
 //********************************************
@@ -736,9 +734,9 @@ void Client::demandAttentionKNotify()
 
 // TODO I probably shouldn't be lazy here and do it without the macro, so that people can read it
 KWIN_COMPARE_PREDICATE(SameApplicationActiveHackPredicate, Client, const Client*,
-                       // ignore already existing splashes, toolbars, utilities, menus and topmenus,
+                       // ignore already existing splashes, toolbars, utilities and menus,
                        // as the app may show those before the main window
-                       !cl->isSplash() && !cl->isToolbar() && !cl->isTopMenu() && !cl->isUtility() && !cl->isMenu()
+                       !cl->isSplash() && !cl->isToolbar() && !cl->isUtility() && !cl->isMenu()
                        && Client::belongToSameApplication(cl, value, true) && cl != value);
 
 Time Client::readUserTimeMapTimestamp(const KStartupInfoId* asn_id, const KStartupInfoData* asn_data,
