@@ -33,8 +33,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <iomanip>
 #include <ios>
 
-#ifdef KWIN_HAVE_OPENGL
-
 namespace KWin
 {
 
@@ -724,8 +722,17 @@ void GLPlatform::detect()
         m_limitedGLSL = m_supportsGLSL && m_chipClass < I965;
     }
 
+#ifdef KWIN_HAVE_OPENGLES
+    if (isMesaDriver()) {
+        // According to the reference implementation in
+        // mesa/demos/src/egl/opengles1/texture_from_pixmap
+        // the mesa egl implementation does not require a strict binding (so far).
+        m_looseBinding = true;
+    }
+#endif
+
     // Loose binding is broken with Gallium drivers in Mesa 7.10
-    if (isGalliumDriver() && mesaVersion() >= kVersionNumber(7, 10))
+    if (isGalliumDriver() && mesaVersion() == kVersionNumber(7, 10, 0))
         m_looseBinding = false;
 
     if (isSoftwareEmulation()) {
@@ -878,5 +885,3 @@ bool GLPlatform::isSoftwareEmulation() const
 
 } // namespace KWin
 
-#endif // KWIN_HAVE_OPENGL
-
