@@ -67,7 +67,6 @@ Sources and other compositing managers:
 */
 
 #include "scene_opengl.h"
-#include "kwinglplatform.h"
 
 #include <kxerrorhandler.h>
 
@@ -77,6 +76,7 @@ Sources and other compositing managers:
 #include "client.h"
 #include "deleted.h"
 #include "effects.h"
+#include "overlaywindow.h"
 
 #include <math.h>
 
@@ -84,8 +84,6 @@ Sources and other compositing managers:
 // normally only few of them are enabled
 //#define CHECK_GL_ERROR
 
-#ifdef KWIN_HAVE_OPENGL_COMPOSITING
-
 #include <X11/extensions/Xcomposite.h>
 
 #include <qpainter.h>
@@ -213,12 +211,15 @@ void SceneOpenGL::windowAdded(Toplevel* c)
 {
     assert(!windows.contains(c));
     windows[ c ] = new Window(c);
+    connect(c, SIGNAL(opacityChanged(KWin::Toplevel*,qreal)), SLOT(windowOpacityChanged(KWin::Toplevel*)));
+    connect(c, SIGNAL(geometryShapeChanged(KWin::Toplevel*,QRect)), SLOT(windowGeometryShapeChanged(KWin::Toplevel*)));
+    connect(c, SIGNAL(windowClosed(KWin::Toplevel*,KWin::Deleted*)), SLOT(windowClosed(KWin::Toplevel*,KWin::Deleted*)));
     c->effectWindow()->setSceneWindow(windows[ c ]);
     c->getShadow();
     windows[ c ]->updateShadow(c->shadow());
 }
 
-void SceneOpenGL::windowClosed(Toplevel* c, Deleted* deleted)
+void SceneOpenGL::windowClosed(KWin::Toplevel* c, KWin::Deleted* deleted)
 {
     assert(windows.contains(c));
     if (deleted != NULL) {
@@ -242,7 +243,7 @@ void SceneOpenGL::windowDeleted(Deleted* c)
     c->effectWindow()->setSceneWindow(NULL);
 }
 
-void SceneOpenGL::windowGeometryShapeChanged(Toplevel* c)
+void SceneOpenGL::windowGeometryShapeChanged(KWin::Toplevel* c)
 {
     if (!windows.contains(c))    // this is ok, shape is not valid
         return;                 // by default
@@ -251,7 +252,7 @@ void SceneOpenGL::windowGeometryShapeChanged(Toplevel* c)
     w->checkTextureSize();
 }
 
-void SceneOpenGL::windowOpacityChanged(Toplevel*)
+void SceneOpenGL::windowOpacityChanged(KWin::Toplevel* t)
 {
 #if 0 // not really needed, windows are painted on every repaint
     // and opacity is used when applying texture, not when
@@ -267,55 +268,43 @@ void SceneOpenGL::windowOpacityChanged(Toplevel*)
 // SceneOpenGL::Texture
 //****************************************
 
-SceneOpenGL::Texture::Texture() : GLTexture()
+SceneOpenGL::Texture::Texture() : GLTexture(*new TexturePrivate())
 {
-    init();
 }
 
-SceneOpenGL::Texture::Texture(const Pixmap& pix, const QSize& size, int depth) : GLTexture()
+SceneOpenGL::Texture::Texture(TexturePrivate& dd) : GLTexture(dd)
+{
+}
+
+SceneOpenGL::Texture::Texture(const SceneOpenGL::Texture& tex) : GLTexture(*tex.d_ptr)
+{
+}
+
+SceneOpenGL::Texture::Texture(const Pixmap& pix, const QSize& size, int depth)
+    : GLTexture(*new TexturePrivate())
 {
-    init();
     load(pix, size, depth);
 }
 
 SceneOpenGL::Texture::Texture(const QPixmap& pix, GLenum target)
-    : GLTexture()
+    : GLTexture(*new TexturePrivate())
 {
-    init();
     load(pix, target);
 }
 
 SceneOpenGL::Texture::~Texture()
 {
-    discard();
 }
 
-void SceneOpenGL::Texture::createTexture()
+SceneOpenGL::Texture& SceneOpenGL::Texture::operator = (const SceneOpenGL::Texture& tex)
 {
-    glGenTextures(1, &mTexture);
+    d_ptr = tex.d_ptr;
+    return *this;
 }
 
 void SceneOpenGL::Texture::discard()
 {
-    if (mTexture != None)
-        release();
-    GLTexture::discard();
-}
-
-QRegion SceneOpenGL::Texture::optimizeBindDamage(const QRegion& reg, int limit)
-{
-    if (reg.rects().count() <= 1)
-        return reg;
-    // try to reduce the number of rects, as especially with SHM mode every rect
-    // causes X roundtrip, even for very small areas - so, when the size difference
-    // between all the areas and the bounding rectangle is small, simply use
-    // only the bounding rectangle
-    int size = 0;
-    foreach (const QRect & r, reg.rects())
-    size += r.width() * r.height();
-    if (reg.boundingRect().width() * reg.boundingRect().height() - size < limit)
-        return reg.boundingRect();
-    return reg;
+    d_ptr = new TexturePrivate();
 }
 
 bool SceneOpenGL::Texture::load(const Pixmap& pix, const QSize& size,
@@ -336,6 +325,7 @@ bool SceneOpenGL::Texture::load(const QImage& image, GLenum target)
 
 bool SceneOpenGL::Texture::load(const QPixmap& pixmap, GLenum target)
 {
+    Q_D(Texture);
     if (pixmap.isNull())
         return false;
 
@@ -344,8 +334,6 @@ bool SceneOpenGL::Texture::load(const QPixmap& pixmap, GLenum target)
         return GLTexture::load(pixmap.toImage(), target);
     }
 
-    y_inverted = true;
-
     // use the X11 pixmap provided by Qt
     return load(pixmap.handle(), pixmap.size(), pixmap.depth());
 }
@@ -372,19 +360,22 @@ SceneOpenGL::Window::~Window()
 // Bind the window pixmap to an OpenGL texture.
 bool SceneOpenGL::Window::bindTexture()
 {
-#ifndef KWIN_HAVE_OPENGLES
-    if (texture.texture() != None && toplevel->damage().isEmpty()) {
-        // texture doesn't need updating, just bind it
-        glBindTexture(texture.target(), texture.texture());
+    if (!texture.isNull()) {
+        if (!toplevel->damage().isEmpty()) {
+            // mipmaps need to be updated
+            texture.setDirty();
+            toplevel->resetDamage(QRect(toplevel->clientPos(), toplevel->clientSize()));
+        }
         return true;
     }
-#endif
     // Get the pixmap with the window contents
     Pixmap pix = toplevel->windowPixmap();
     if (pix == None)
         return false;
+
     bool success = texture.load(pix, toplevel->size(), toplevel->depth(),
                                 toplevel->damage());
+
     if (success)
         toplevel->resetDamage(QRect(toplevel->clientPos(), toplevel->clientSize()));
     else
@@ -416,7 +407,7 @@ void SceneOpenGL::Window::checkTextureSize()
 // when the window's composite pixmap is discarded, undo binding it to the texture
 void SceneOpenGL::Window::pixmapDiscarded()
 {
-    texture.release();
+    texture.discard();
 }
 
 QMatrix4x4 SceneOpenGL::Window::transformation(int mask, const WindowPaintData &data) const
@@ -466,13 +457,32 @@ void SceneOpenGL::Window::performPaint(int mask, QRegion region, WindowPaintData
             return; // Only painting translucent and window is opaque
         }*/
 
-    // Intersect the clip region with the rectangle the window occupies on the screen
-    if (!(mask & (PAINT_WINDOW_TRANSFORMED | PAINT_SCREEN_TRANSFORMED)))
-        region &= toplevel->visibleRect();
-
     if (region.isEmpty())
         return;
 
+    if (region != infiniteRegion() && !(mask & PAINT_WINDOW_TRANSFORMED)) {
+        WindowQuadList quads;
+        const QRegion filterRegion = region.translated(-x(), -y());
+        // split all quads in bounding rect with the actual rects in the region
+        foreach (const WindowQuad &quad, data.quads) {
+            foreach (const QRect &r, filterRegion.rects()) {
+                const QRectF rf(r);
+                const QRectF quadRect(QPointF(quad.left(), quad.top()), QPointF(quad.right(), quad.bottom()));
+                // case 1: completely contains, include and do not check other rects
+                if (rf.contains(quadRect)) {
+                    quads << quad;
+                    break;
+                }
+                // case 2: intersection
+                if (rf.intersects(quadRect)) {
+                    const QRectF intersected = rf.intersected(quadRect);
+                    quads << quad.makeSubQuad(intersected.left(), intersected.top(), intersected.right(), intersected.bottom());
+                }
+            }
+        }
+        data.quads = quads;
+    }
+
     if (!bindTexture())
         return;
 
@@ -601,8 +611,6 @@ void SceneOpenGL::Window::performPaint(int mask, QRegion region, WindowPaintData
 
 void SceneOpenGL::Window::paintDecoration(const QPixmap* decoration, TextureType decorationType, const QRegion& region, const QRect& rect, const WindowPaintData& data, const WindowQuadList& quads, bool updateDeco)
 {
-    if (quads.isEmpty())
-        return;
     SceneOpenGL::Texture* decorationTexture;
     switch(decorationType) {
     case DecorationTop:
@@ -622,7 +630,7 @@ void SceneOpenGL::Window::paintDecoration(const QPixmap* decoration, TextureType
     }
     if (decorationTexture->texture() != None && !updateDeco) {
         // texture doesn't need updating, just bind it
-        glBindTexture(decorationTexture->target(), decorationTexture->texture());
+        decorationTexture->bind();
     } else if (!decoration->isNull()) {
         bool success = decorationTexture->load(*decoration);
         if (!success) {
@@ -631,6 +639,16 @@ void SceneOpenGL::Window::paintDecoration(const QPixmap* decoration, TextureType
         }
     } else
         return;
+
+    // We have to update the texture although we do not paint anything.
+    // This is especially needed if we draw the opaque part of the window
+    // and the decoration in two different passes (as we in Scene::paintSimpleWindow do).
+    // Otherwise we run into the situation that in the first pass there are some
+    // pending decoration repaints but we don't paint the decoration and in the
+    // second pass it's the other way around.
+    if (quads.isEmpty())
+        return;
+
     if (filter == ImageFilterGood)
         decorationTexture->setFilter(GL_LINEAR);
     else
@@ -662,6 +680,9 @@ void SceneOpenGL::Window::paintShadow(const QRegion &region, const WindowPaintDa
     quads.append(data.quads.select(WindowQuadShadowBottom));
     quads.append(data.quads.select(WindowQuadShadowBottomLeft));
     quads.append(data.quads.select(WindowQuadShadowLeft));
+    if (quads.isEmpty()) {
+        return;
+    }
     GLTexture *texture = static_cast<SceneOpenGLShadow*>(m_shadow)->shadowTexture();
     if (!texture) {
         return;
@@ -713,7 +734,7 @@ void SceneOpenGL::Window::makeDecorationArrays(const WindowQuadList& quads, cons
         vertices << quad[ 1 ].x();
         vertices << quad[ 1 ].y();
 
-        if (tex->getYInverted()) {
+        if (tex->isYInverted()) {
             texcoords << (float)(quad.originalRight() - rect.x()) / width;
             texcoords << (float)(quad.originalTop() - rect.y()) / height;
             texcoords << (float)(quad.originalLeft() - rect.x()) / width;
@@ -1708,5 +1729,3 @@ bool SceneOpenGLShadow::prepareBackend()
 }
 
 } // namespace
-
-#endif
