@@ -40,6 +40,11 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <KServiceTypeTrader>
 #include <KStandardDirs>
 #include <KDesktopFile>
+#include <KRun>
+
+#include <QtCore/QMap>
+#include <QtGui/QHelpEvent>
+#include <QtGui/QToolTip>
 
 namespace TaskManager
 {
@@ -50,33 +55,36 @@ QAction *standardGroupableAction(GroupableAction action, AbstractGroupableItem *
     Q_ASSERT(item);
 
     switch (action) {
-        case MaximizeAction:
-            return new MaximizeActionImpl(parent, item);
-            break;
-        case MinimizeAction:
-            return new MinimizeActionImpl(parent, item);
-            break;
-        case ToCurrentDesktopAction:
-            return new ToCurrentDesktopActionImpl(parent, item);
-            break;
-        case ToDesktopAction:
-            return new ToDesktopActionImpl(parent, item, desktop);
-            break;
-        case ShadeAction:
-            return new ShadeActionImpl(parent, item);
-            break;
-        case CloseAction:
-            return new CloseActionImpl(parent, item);
-            break;
-        case ViewFullscreenAction:
-            return new ViewFullscreenActionImpl(parent, item);
-            break;
-        case KeepBelowAction:
-            return new KeepBelowActionImpl(parent, item);
-            break;
-        case ToggleLauncherAction:
-            return new ToggleLauncherActionImpl(parent, item, strategy);
-            break;
+    case MaximizeAction:
+        return new MaximizeActionImpl(parent, item);
+        break;
+    case MinimizeAction:
+        return new MinimizeActionImpl(parent, item);
+        break;
+    case ToCurrentDesktopAction:
+        return new ToCurrentDesktopActionImpl(parent, item);
+        break;
+    case ToDesktopAction:
+        return new ToDesktopActionImpl(parent, item, desktop);
+        break;
+    case ShadeAction:
+        return new ShadeActionImpl(parent, item);
+        break;
+    case CloseAction:
+        return new CloseActionImpl(parent, item);
+        break;
+    case ViewFullscreenAction:
+        return new ViewFullscreenActionImpl(parent, item);
+        break;
+    case KeepBelowAction:
+        return new KeepBelowActionImpl(parent, item);
+        break;
+    case ToggleLauncherAction:
+        return new ToggleLauncherActionImpl(parent, item, strategy);
+        break;
+    case NewInstanceAction:
+        return new NewInstanceActionImpl(parent, item);
+        break;
     }
 
     return 0;
@@ -87,12 +95,12 @@ QAction* standardTaskAction(TaskAction action, TaskItem *item, QObject *parent)
     Q_ASSERT(item);
 
     switch (action) {
-        case ResizeAction:
-            return new ResizeActionImpl(parent, item);
-            break;
-        case MoveAction:
-            return new MoveActionImpl(parent, item);
-            break;
+    case ResizeAction:
+        return new ResizeActionImpl(parent, item);
+        break;
+    case MoveAction:
+        return new MoveActionImpl(parent, item);
+        break;
     }
 
     return 0;
@@ -104,14 +112,55 @@ QAction* standardGroupingAction(GroupingAction action, AbstractGroupableItem *it
     Q_ASSERT(strategy);
 
     switch (action) {
-        case LeaveGroupAction:
-            return new LeaveGroupActionImpl(parent, item, strategy);
-            break;
+    case LeaveGroupAction:
+        return new LeaveGroupActionImpl(parent, item, strategy);
+        break;
     }
 
     return 0;
 }
 
+static QString qt_strippedText(QString s)
+{
+    s.remove(QString::fromLatin1("..."));
+    int i = 0;
+    while (i < s.size()) {
+        ++i;
+        if (s.at(i - 1) != QLatin1Char('&')) {
+            continue;
+        }
+
+        if (i < s.size() && s.at(i) == QLatin1Char('&')) {
+            ++i;
+        }
+        s.remove(i - 1, 1);
+    }
+    return s.trimmed();
+}
+
+ToolTipMenu::ToolTipMenu(QWidget *parent, const QString &title)
+    : QMenu(title, parent)
+{
+}
+
+bool ToolTipMenu::event(QEvent* e)
+{
+    if (QEvent::ToolTip == e->type()) {
+        //show action tooltip instead of widget tooltip
+        QHelpEvent *he = dynamic_cast<QHelpEvent*>(e);
+        QAction *act = he ? actionAt(he->pos()) : 0;
+        if (act) {
+            if (qt_strippedText(act->text()) != act->toolTip()) {
+                QToolTip::showText(he->globalPos(), act->toolTip(), this);
+            } else {
+                QToolTip::hideText();
+            }
+            return true;
+        }
+    }
+    return QMenu::event(e);
+}
+
 MinimizeActionImpl::MinimizeActionImpl(QObject *parent, AbstractGroupableItem *item)
     : QAction(parent)
 {
@@ -146,8 +195,7 @@ ShadeActionImpl::ShadeActionImpl(QObject *parent, AbstractGroupableItem *item)
 ResizeActionImpl::ResizeActionImpl(QObject *parent, TaskItem* item)
     : QAction(parent)
 {
-    TaskPtr task = item->task();
-    connect(this, SIGNAL(triggered()), task.data(), SLOT(resize()));
+    connect(this, SIGNAL(triggered()), item->task(), SLOT(resize()));
     setText(i18n("Re&size"));
     setEnabled(item->isActionSupported(NET::ActionResize));
 }
@@ -155,8 +203,7 @@ ResizeActionImpl::ResizeActionImpl(QObject *parent, TaskItem* item)
 MoveActionImpl::MoveActionImpl(QObject *parent, TaskItem* item)
     : QAction(parent)
 {
-    TaskPtr task = item->task();
-    connect(this, SIGNAL(triggered()), task.data(), SLOT(move()));
+    connect(this, SIGNAL(triggered()), item->task(), SLOT(move()));
     setText(i18n("&Move"));
     setIcon(KIcon("transform-move"));
     setEnabled(item->isActionSupported(NET::ActionMove));
@@ -180,12 +227,12 @@ AbstractGroupableItemAction::AbstractGroupableItemAction(QObject *parent, Abstra
         addToTasks(group);
     } else if (TaskItem *taskItem = qobject_cast<TaskItem *>(item)) {
         m_tasks.append(taskItem->task());
-    } 
+    }
 }
 
 void AbstractGroupableItemAction::addToTasks(TaskGroup *group)
 {
-    foreach (AbstractGroupableItem *item, group->members()) {
+    foreach (AbstractGroupableItem * item, group->members()) {
         TaskGroup *subGroup = qobject_cast<TaskGroup *>(item);
         if (subGroup) {
             addToTasks(subGroup);
@@ -203,16 +250,16 @@ ToCurrentDesktopActionImpl::ToCurrentDesktopActionImpl(QObject *parent, Abstract
     setEnabled(!item->isOnCurrentDesktop() && item->isActionSupported(NET::ActionChangeDesktop));
 }
 
-void ToCurrentDesktopActionImpl::slotToCurrentDesktop() 
+void ToCurrentDesktopActionImpl::slotToCurrentDesktop()
 {
     const int desktop = KWindowSystem::currentDesktop();
-    foreach (TaskPtr task, m_tasks) {
-        task->toDesktop(desktop);
+    foreach (QWeakPointer<Task> task, m_tasks) {
+        if (task) {
+            task.data()->toDesktop(desktop);
+        }
     }
 }
 
-
-
 ToDesktopActionImpl::ToDesktopActionImpl(QObject *parent, AbstractGroupableItem *item, int desktop)
     : AbstractGroupableItemAction(parent, item),
       m_desktop(desktop)
@@ -230,23 +277,25 @@ ToDesktopActionImpl::ToDesktopActionImpl(QObject *parent, AbstractGroupableItem
 
 }
 
-void ToDesktopActionImpl::slotToDesktop() 
+void ToDesktopActionImpl::slotToDesktop()
 {
-    foreach (TaskPtr task, m_tasks) {
-        task->toDesktop(m_desktop);
+    foreach (QWeakPointer<Task> task, m_tasks) {
+        if (task) {
+            task.data()->toDesktop(m_desktop);
+        }
     }
 }
 
 
-
 DesktopsMenu::DesktopsMenu(QWidget *parent, AbstractGroupableItem *item)
-    : QMenu(parent)
+    : ToolTipMenu(parent)
 {
-    setTitle( i18n("Move To &Desktop") );
-    addAction( new ToDesktopActionImpl(this,item,0) );      //0 means all desktops
+    setTitle(i18n("Move To &Desktop"));
+    addAction(new ToCurrentDesktopActionImpl(this, item));
+    addAction(new ToDesktopActionImpl(this, item, 0));      //0 means all desktops
     addSeparator();
     for (int i = 1; i <= TaskManager::self()->numberOfDesktops(); i++) {
-        addAction( new ToDesktopActionImpl(this,item,i) );
+        addAction(new ToDesktopActionImpl(this, item, i));
     }
     setEnabled(item->isActionSupported(NET::ActionChangeDesktop));
 }
@@ -283,20 +332,23 @@ ViewFullscreenActionImpl::ViewFullscreenActionImpl(QObject *parent, AbstractGrou
 }
 
 AdvancedMenu::AdvancedMenu(QWidget *parent, AbstractGroupableItem *item, GroupManager *strategy)
-    : QMenu(parent)
+    : ToolTipMenu(parent)
 {
-    setTitle(i18n("Ad&vanced"));
+    setTitle(i18n("More Actions"));
+    if (item->itemType() == TaskItemType) {
+        addAction(new MoveActionImpl(this, static_cast<TaskItem *>(item)));
+        addAction(new ResizeActionImpl(this, static_cast<TaskItem *>(item)));
+    }
     addAction(new KeepAboveActionImpl(this, item));
     addAction(new KeepBelowActionImpl(this, item));
     addAction(new ViewFullscreenActionImpl(this, item));
+    addAction(new ShadeActionImpl(this, item));
 
-    addSeparator();
-
-    addAction(new ToggleLauncherActionImpl(this, item, strategy));
     if (strategy->taskGrouper()) {
         QList<QAction*> groupingStrategyActions = strategy->taskGrouper()->strategyActions(this, item);
         if (!groupingStrategyActions.isEmpty()) {
-            foreach (QAction *action, groupingStrategyActions) {
+            addSeparator();
+            foreach (QAction * action, groupingStrategyActions) {
                 addAction(action);
             }
             // delete groupingStrategyActions;
@@ -316,30 +368,34 @@ LeaveGroupActionImpl::LeaveGroupActionImpl(QObject *parent, AbstractGroupableIte
 
 void LeaveGroupActionImpl::leaveGroup()
 {
-    groupingStrategy->manualGroupingRequest(abstractItem,abstractItem->parentGroup()->parentGroup());
+    groupingStrategy->manualGroupingRequest(abstractItem, abstractItem->parentGroup()->parentGroup());
 }
 
 ToggleLauncherActionImpl::ToggleLauncherActionImpl(QObject *parent, AbstractGroupableItem *item, GroupManager *strategy)
     : QAction(parent), m_abstractItem(item), m_groupingStrategy(strategy)
 {
-    connect(this, SIGNAL(triggered()), this, SLOT(toggleLauncher()));
+    if (strategy->launchersLocked()) {
+        setVisible(false);
+        setChecked(false);
+    } else {
+        m_url = m_abstractItem->launcherUrl();
+        connect(this, SIGNAL(triggered()), this, SLOT(toggleLauncher()));
 
-    switch (item->itemType()) {
+        switch (m_abstractItem->itemType()) {
         case LauncherItemType:
-            m_name = item->name();
             setText(i18n("Remove This Launcher"));
             break;
 
         case GroupItemType: {
-            TaskGroup *group = static_cast<TaskGroup *>(item);
-            foreach (AbstractGroupableItem *i, group->members()) {
-                if (i->itemType() != GroupItemType) {
-                    item = i;
+            TaskGroup *group = static_cast<TaskGroup *>(m_abstractItem);
+            foreach (AbstractGroupableItem * i, group->members()) {
+                if (TaskItemType == i->itemType()) {
+                    m_abstractItem = i;
                     break;
                 }
             }
 
-            if (item->itemType() == GroupItemType) {
+            if (TaskItemType != m_abstractItem->itemType()) {
                 setVisible(false);
                 setChecked(false);
                 break;
@@ -348,36 +404,96 @@ ToggleLauncherActionImpl::ToggleLauncherActionImpl(QObject *parent, AbstractGrou
         } // fallthrough to TaskItemType below
 
         case TaskItemType:
-            m_name = static_cast<TaskItem *>(item)->task()->classClass();
-            setText(i18n("&Show A Launcher For %1 When It Is Not Running", m_name));
+            setText(i18n("&Show A Launcher When Not Running"));
             setCheckable(true);
             break;
-    }
-
-    m_url = item->launcherUrl();
-    if (m_url.isEmpty()) {
-        //don't show the possibility to add a launcher if we don't have a url for it
-        //kDebug() << "No executable found for" << m_name;
-        setVisible(false);
-        setChecked(false);
-    } else {
+        }
         setChecked(m_groupingStrategy->launcherExists(m_url));
     }
 }
 
 void ToggleLauncherActionImpl::toggleLauncher()
+{
+    if (!m_url.isValid()) {
+        // No valid desktop file found, so prompt user...
+        AppSelectorDialog *dlg = new AppSelectorDialog(m_abstractItem, m_groupingStrategy);
+        dlg->show();
+        return;
+    } else if (m_groupingStrategy->launcherExists(m_url)) {
+        m_groupingStrategy->removeLauncher(m_url);
+    } else if (m_url.isLocalFile() && KDesktopFile::isDesktopFile(m_url.toLocalFile())) {
+        m_groupingStrategy->addLauncher(m_url, QIcon(), QString(), QString(),
+                                        static_cast<TaskItem *>(m_abstractItem)->task()
+                                        ? static_cast<TaskItem *>(m_abstractItem)->task()->classClass()
+                                        : QString());
+    }
+}
+
+AppSelectorDialog::AppSelectorDialog(AbstractGroupableItem* item, GroupManager* strategy)
+    : KOpenWithDialog(KUrl::List(), i18n("The application, to which this task is associated with, could not be determined. "
+                                         "Please select the appropriate application from the list below:"), QString(), 0L),
+    m_abstractItem(item),
+    m_groupingStrategy(strategy)
+{
+    hideNoCloseOnExit();
+    hideRunInTerminal();
+    setAttribute(Qt::WA_DeleteOnClose);
+    setWindowModality(Qt::WindowModal);
+    connect(this, SIGNAL(accepted()), SLOT(launcherSelected()));
+}
+
+void AppSelectorDialog::launcherSelected()
+{
+    if (m_abstractItem && m_groupingStrategy) {
+        KService::Ptr srv = service();
+        TaskItem *taskItem = static_cast<TaskItem *>(m_abstractItem.data());
+        QString wmClass = taskItem->task() ? taskItem->task()->classClass() : QString();
+
+        if (srv && srv->isApplication() && !srv->entryPath().isEmpty()) {
+            KUrl url = KUrl::fromPath(srv->entryPath());
+
+            if (url.isLocalFile() && KDesktopFile::isDesktopFile(url.toLocalFile())) {
+                taskItem->setLauncherUrl(url);
+                m_groupingStrategy->addLauncher(url, QIcon(), QString(), QString(), wmClass);
+            }
+        } else {
+            QString path = text();
+
+            if (!path.isEmpty()) {
+                KUrl url = KUrl::fromPath(path);
+                if (url.isLocalFile()) {
+                    taskItem->setLauncherUrl(url);
+                    m_groupingStrategy->addLauncher(url, taskItem->icon(), taskItem->name(), wmClass, wmClass);
+                }
+            }
+        }
+    }
+}
+
+NewInstanceActionImpl::NewInstanceActionImpl(QObject *parent, AbstractGroupableItem *item)
+    : QAction(parent), m_abstractItem(item)
+{
+    if (LauncherItemType == item->itemType()) {
+        setVisible(false);
+    } else {
+        setIcon(KIcon("system-run"));
+        setText(i18n("Start New Instance"));
+
+        connect(this, SIGNAL(triggered()), this, SLOT(launchNewInstance()));
+        m_url = item->launcherUrl();
+        if (m_url.isEmpty()) {
+            setVisible(false);
+        }
+    }
+}
+
+void NewInstanceActionImpl::launchNewInstance()
 {
     if (!m_url.isValid()) {
         return;
     }
 
-    if (m_groupingStrategy->launcherExists(m_url)) {
-        m_groupingStrategy->removeLauncher(m_url);
-    } else if (m_url.isLocalFile() && KDesktopFile::isDesktopFile(m_url.toLocalFile())) {
-        m_groupingStrategy->addLauncher(m_url);
-    } else {
-        m_groupingStrategy->addLauncher(m_url, m_abstractItem->icon(), m_name);
-    }
+    new KRun(m_url, 0);
 }
 
 EditGroupActionImpl::EditGroupActionImpl(QObject *parent, TaskGroup *group, GroupManager *groupManager)
@@ -399,7 +515,7 @@ EditGroupActionImpl::EditGroupActionImpl(QObject *parent, TaskGroup *group, Grou
 }
 
 GroupingStrategyMenu::GroupingStrategyMenu(QWidget *parent, AbstractGroupableItem* item, GroupManager *strategy)
-    : QMenu(parent)
+    : ToolTipMenu(parent)
 {
     Q_ASSERT(item);
     Q_ASSERT(strategy);
@@ -409,7 +525,7 @@ GroupingStrategyMenu::GroupingStrategyMenu(QWidget *parent, AbstractGroupableIte
         QList<QAction*> groupingStrategyActions = strategy->taskGrouper()->strategyActions(this, item);
         if (!groupingStrategyActions.empty()) {
             addSeparator();
-            foreach (QAction *action, groupingStrategyActions) {
+            foreach (QAction * action, groupingStrategyActions) {
                 addAction(action);
             }
         }
@@ -417,29 +533,36 @@ GroupingStrategyMenu::GroupingStrategyMenu(QWidget *parent, AbstractGroupableIte
 }
 
 
-BasicMenu::BasicMenu(QWidget *parent, TaskItem* item, GroupManager *strategy, QList<QAction *> visualizationActions)
-    : QMenu(parent)
+BasicMenu::BasicMenu(QWidget *parent, TaskItem* item, GroupManager *strategy, QList<QAction *> visualizationActions, QList <QAction*> appActions)
+    : ToolTipMenu(parent)
 {
     Q_ASSERT(item);
     Q_ASSERT(strategy);
 
     setTitle(item->name());
     setIcon(item->icon());
+    if (appActions.count()) {
+        foreach (QAction * action, appActions) {
+            if (action->isSeparator()) {
+                addSeparator();
+            } else {
+                addAction(action);
+            }
+        }
+        addSeparator();
+    }
 
     if (TaskManager::self()->numberOfDesktops() > 1) {
         addMenu(new DesktopsMenu(this, item));
-        addAction(new ToCurrentDesktopActionImpl(this, item));
     }
 
-    addAction(new MoveActionImpl(this, item));
-    addAction(new ResizeActionImpl(this, item));
     addAction(new MinimizeActionImpl(this, item));
     addAction(new MaximizeActionImpl(this, item));
-    addAction(new ShadeActionImpl(this, item));
-
+    addAction(new NewInstanceActionImpl(this, item));
+    addAction(new ToggleLauncherActionImpl(this, item, strategy));
     addMenu(new AdvancedMenu(this, item, strategy));
 
-    foreach (QAction *action, visualizationActions) {
+    foreach (QAction * action, visualizationActions) {
         addAction(action);
     }
 
@@ -447,37 +570,46 @@ BasicMenu::BasicMenu(QWidget *parent, TaskItem* item, GroupManager *strategy, QL
     addAction(new CloseActionImpl(this, item));
 }
 
-BasicMenu::BasicMenu(QWidget *parent, TaskGroup* group, GroupManager *strategy, QList <QAction*> visualizationActions)
-    : QMenu(parent)
+BasicMenu::BasicMenu(QWidget *parent, TaskGroup* group, GroupManager *strategy, QList <QAction*> visualizationActions, QList <QAction*> appActions)
+    : ToolTipMenu(parent)
 {
     Q_ASSERT(group);
     Q_ASSERT(strategy);
 
     setTitle(group->name());
     setIcon(group->icon());
-    foreach (AbstractGroupableItem *item, group->members()) {
+    if (appActions.count()) {
+        foreach (QAction * action, appActions) {
+            if (action->isSeparator()) {
+                addSeparator();
+            } else {
+                addAction(action);
+            }
+        }
+        addSeparator();
+    }
+
+    foreach (AbstractGroupableItem * item, group->members()) {
         if (item->itemType() == GroupItemType) {
             addMenu(new BasicMenu(this, dynamic_cast<TaskGroup*>(item), strategy));
         } else {
             addMenu(new BasicMenu(this, dynamic_cast<TaskItem*>(item), strategy));
         }
     }
-
     addSeparator();
 
     if (TaskManager::self()->numberOfDesktops() > 1) {
         addMenu(new DesktopsMenu(this, group));
-        addAction(new ToCurrentDesktopActionImpl(this, group));
     }
 
     addAction(new MinimizeActionImpl(this, group));
     addAction(new MaximizeActionImpl(this, group));
-    addAction(new ShadeActionImpl(this, group));
-
+    addAction(new NewInstanceActionImpl(this, group));
+    addAction(new ToggleLauncherActionImpl(this, group, strategy));
     addMenu(new AdvancedMenu(this, group, strategy));
     addAction(new EditGroupActionImpl(this, group, strategy));
 
-    foreach (QAction *action, visualizationActions) {
+    foreach (QAction * action, visualizationActions) {
         addAction(action);
     }
 
@@ -485,42 +617,56 @@ BasicMenu::BasicMenu(QWidget *parent, TaskGroup* group, GroupManager *strategy,
     addAction(new CloseActionImpl(this, group));
 }
 
-BasicMenu::BasicMenu(QWidget *parent, LauncherItem* item, GroupManager *strategy, QList<QAction *> visualizationActions)
-    : QMenu(parent)
+BasicMenu::BasicMenu(QWidget *parent, LauncherItem* item, GroupManager *strategy, QList<QAction *> visualizationActions, QList <QAction*> appActions)
+    : ToolTipMenu(parent)
 {
     Q_ASSERT(item);
     Q_ASSERT(strategy);
 
     setTitle(item->name());
     setIcon(item->icon());
+    if (appActions.count()) {
+        foreach (QAction * action, appActions) {
+            if (action->isSeparator()) {
+                addSeparator();
+            } else {
+                addAction(action);
+            }
+        }
+        addSeparator();
+    }
 
     addAction(new ToggleLauncherActionImpl(this, item, strategy));
 
-    addSeparator();
-
-    foreach (QAction *action, visualizationActions) {
-        addAction(action);
+    if (!visualizationActions.isEmpty()) {
+        addSeparator();
+        foreach (QAction * action, visualizationActions) {
+            addAction(action);
+        }
     }
 }
 
 GroupPopupMenu::GroupPopupMenu(QWidget *parent, TaskGroup *group, GroupManager *groupManager)
-    :QMenu(parent)
+    : ToolTipMenu(parent)
 {
     setTitle(group->name());
     setIcon(group->icon());
-    foreach (AbstractGroupableItem *item, group->members()) {
+    foreach (AbstractGroupableItem * item, group->members()) {
         if (!item) {
             kDebug() << "invalid Item";
             continue;
         }
 
         if (item->itemType() == GroupItemType) {
-            QMenu* menu = new GroupPopupMenu (this, qobject_cast<TaskGroup*>(item), groupManager);
+            ToolTipMenu* menu = new GroupPopupMenu(this, qobject_cast<TaskGroup*>(item), groupManager);
             addMenu(menu);
         } else {
-            QAction* action = new QAction(item->icon(), item->name(), this);
-            connect(action, SIGNAL(triggered(bool)), (qobject_cast<TaskItem*>(item))->task().data() , SLOT(activateRaiseOrIconify()));
-            addAction(action);
+            TaskItem *taskItem = qobject_cast<TaskItem*>(item);
+            if (taskItem && taskItem->task()) {
+                QAction* action = new QAction(item->icon(), item->name(), this);
+                connect(action, SIGNAL(triggered(bool)), taskItem->task(), SLOT(activateRaiseOrIconify()));
+                addAction(action);
+            }
         }
     }
 }
